from django.contrib.auth import authenticate, login, logout as django_logout
from django.contrib.auth import get_user_model
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponseBadRequest

from oauth2client.client import OAuth2WebServerFlow
from oauth2client import xsrfutil
from oauth2client.django_orm import Storage

from google_oauth.models import GoogleProfile

import httplib2

User = get_user_model()


def logout(request, redirect_url=settings.LOGOUT_REDIRECT_URL):
    django_logout(request)
    return HttpResponseRedirect(request.build_absolute_uri(redirect_url))


def begin_auth(request):

    # Request an authorization url to send the user to...
    callback_url = request.build_absolute_uri(reverse('google_oauth.views.thanks'))
    flow = OAuth2WebServerFlow(client_id=settings.GOOGLE_OAUTH_CLIENT_ID, client_secret=settings.GOOGLE_OAUTH_CLIENT_SECRET,
                               scope="https://www.googleapis.com/auth/plus.login https://www.googleapis.com/auth/plus.profile.emails.read", redirect_uri=callback_url)

    flow.params['state'] = xsrfutil.generate_token(settings.GOOGLE_OAUTH_CLIENT_SECRET, request.user)

    auth_url = flow.step1_get_authorize_url()

    # Then send them over there, durh.
    return HttpResponseRedirect(auth_url)


def thanks(request, redirect_url=settings.LOGIN_REDIRECT_URL):
    if not xsrfutil.validate_token(settings.GOOGLE_OAUTH_CLIENT_SECRET, request.REQUEST['state'],
                                   request.user):
        return HttpResponseBadRequest()

    callback_url = request.build_absolute_uri(reverse('google_oauth.views.thanks'))
    flow = OAuth2WebServerFlow(client_id=settings.GOOGLE_OAUTH_CLIENT_ID, client_secret=settings.GOOGLE_OAUTH_CLIENT_SECRET,
                               scope="https://www.googleapis.com/auth/plus.login https://www.googleapis.com/auth/plus.profile.emails.read",
                               redirect_uri=callback_url)

    credential = flow.step2_exchange(request.REQUEST)

    from apiclient.discovery import build

    http = httplib2.Http()
    credential.authorize(http)

    plus = build("plus", "v1", http=http)
    guser = plus.people().get(userId="me").execute()

    user = request.user if request.user.pk else None

    if user is None and 'emails' in guser and len(guser['emails']) > 0:
        try:
            user = User.objects.get(email=guser['emails'][0]['value'])
        except User.DoesNotExist:
            user = None
    if user is None:
        username = 'google%s' % guser['id']
        email = guser['emails'][0]['value'] if 'emails' in guser and len(guser['emails']) > 0 else ''
        if 'displayName' in guser:
            username = guser['displayName'].replace(' ', '_')
        user = User.objects.create_user(username, email, request.REQUEST['state'])
        profile = GoogleProfile()
        profile.user = user
        profile.credential = credential
        profile.save()

    user.backend = 'django.contrib.auth.backends.ModelBackend'
    login(request, user)
    return HttpResponseRedirect(redirect_url)



from django.db import models
from django.conf import settings
from oauth2client.django_orm import CredentialsField


class GoogleModel(models.Model):
    def disconnect_twitter(self):
        try:
            tp = GoogleProfile.objects.get(user=self)
            tp.delete()
        except GoogleProfile.DoesNotExist:
            pass

    class Meta:
        abstract = True


class GoogleProfile(models.Model):
    """
        An example Profile model that handles storing the oauth_token and
        oauth_secret in relation to a user. Adapt this if you have a current
        setup, there's really nothing special going on here.
    """
    user = models.OneToOneField(settings.AUTH_USER_MODEL)
    credential = CredentialsField()



#!/usr/bin/env python

import os
import sys

from setuptools import setup
from setuptools import find_packages

__author__ = 'Marco trik Marche <marco.marche@gnucoop.com>'
__version__ = '1.0.2'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

setup(
    name='django-google-oauth',
    version=__version__,
    install_requires=['google-api-python-client', 'django'],
    author='Marco trik Marche',
    author_email='marco.marche@gnucoop.com',
    license=open('LICENSE').read(),
    url='https://bitbucket.org/trik82/django-google-oauth/tree/master',
    keywords='google google+ api auth oauth2 django integration',
    description='Django Integration for Google+ OAuth2',
    long_description=open('README.rst').read(),
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Communications :: Chat',
        'Topic :: Internet'
    ]
)



from django.conf.urls import *

# your_app = name of your root djang app.
urlpatterns = patterns('google_oauth.views',
    # First leg of the authentication journey...
    url(r'^login/?$', "begin_auth", name="google_login"),

    # Logout, if need be
    url(r'^logout/?$', "logout", name="google_logout"),  # Calling logout and what not

    # This is where they're redirected to after authorizing - we'll
    # further (silently) redirect them again here after storing tokens and such.
    url(r'^thanks/?$', "thanks", name="google_callback"),
)
